@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2010 Joe Wreschnig, Michael Urman
 # Copyright 2010,2013 Christoph Reiter
 # Copyright 2013-2015 Nick Boultbee
@@ -15,15 +14,14 @@
 import os
 import re
 from re import Scanner  # type: ignore
+from urllib.parse import quote_plus
 
 from senf import sep, fsnative, expanduser
 
 from quodlibet import util
 from quodlibet.query import Query
-from quodlibet.compat import exec_, itervalues
 from quodlibet.util.path import strip_win32_incompat_from_path, limit_path
 from quodlibet.formats._audio import decode_value, FILESYSTEM_TAGS
-from quodlibet.compat import quote_plus, text_type, number_types
 
 # Token types.
 (OPEN, CLOSE, TEXT, COND, EOF) = range(5)
@@ -41,7 +39,7 @@ class LexerError(error):
     pass
 
 
-class PatternLexeme(object):
+class PatternLexeme:
     _reverse = {OPEN: "OPEN", CLOSE: "CLOSE", TEXT: "TEXT", COND: "COND",
                 EOF: "EOF"}
 
@@ -50,7 +48,7 @@ class PatternLexeme(object):
         self.lexeme = lexeme
 
     def __repr__(self):
-        return (super(PatternLexeme, self).__repr__().split()[0] +
+        return (super().__repr__().split()[0] +
                 " type=" + repr(self.type) + " (" +
                 str(self._reverse[self.type]) +
                 "), lexeme=" + repr(self.lexeme) + ">")
@@ -79,7 +77,7 @@ class PatternLexer(Scanner):
             return iter(s[0] + [PatternLexeme(EOF, "")])
 
 
-class PatternNode(object):
+class PatternNode:
     def __init__(self):
         self.children = []
 
@@ -87,7 +85,7 @@ class PatternNode(object):
         return "Pattern(%s)" % (", ".join(map(repr, self.children)))
 
 
-class TextNode(object):
+class TextNode:
     def __init__(self, text):
         self.text = text
 
@@ -95,7 +93,7 @@ class TextNode(object):
         return "Text(\"%s\")" % self.text
 
 
-class ConditionNode(object):
+class ConditionNode:
     def __init__(self, expr, ifcase, elsecase):
         self.expr = expr
         self.ifcase = ifcase
@@ -106,7 +104,7 @@ class ConditionNode(object):
         return "Condition(expression: \"%s\", if: %s, else: %s)" % (t, i, e)
 
 
-class TagNode(object):
+class TagNode:
     def __init__(self, tag):
         self.tag = tag
 
@@ -114,7 +112,7 @@ class TagNode(object):
         return "Tag(\"%s\")" % self.tag
 
 
-class PatternParser(object):
+class PatternParser:
     def __init__(self, tokens):
         self.tokens = iter(tokens)
         self.lookahead = next(self.tokens)
@@ -136,7 +134,7 @@ class PatternParser(object):
         tag = self.lookahead.lexeme
         # fix bad tied tags
         if tag[:1] != "~" and "~" in tag:
-                tag = "~" + tag
+            tag = "~" + tag
         try:
             self.match(TEXT)
         except ParseError:
@@ -183,7 +181,7 @@ class PatternParser(object):
             self.lookahead = PatternLexeme(EOF, "")
 
 
-class PatternFormatter(object):
+class PatternFormatter:
     _format = None
     _post = None
     _text = None
@@ -209,7 +207,7 @@ class PatternFormatter(object):
         def list_separate(self, key):
             return [u""]
 
-    class SongProxy(object):
+    class SongProxy:
         def __init__(self, realsong, formatter):
             self.__song = realsong
             self.__formatter = formatter
@@ -222,7 +220,7 @@ class PatternFormatter(object):
 
         def comma(self, key):
             value = self.__song.comma(key)
-            if isinstance(value, number_types):
+            if isinstance(value, (int, float)):
                 value = decode_value(key, value)
             if self.__formatter:
                 return self.__formatter(key, value)
@@ -272,7 +270,7 @@ class PatternFormatter(object):
     __mod__ = format
 
 
-class PatternCompiler(object):
+class PatternCompiler:
     def __init__(self, root):
         self.__root = root.node
 
@@ -289,10 +287,10 @@ class PatternCompiler(object):
         content.append("  return r")
         code = "\n".join(content)
 
-        scope = dict(itervalues(queries))
+        scope = dict(queries.values())
         if text_formatter:
             scope["_format"] = text_formatter
-        exec_(compile(code, "<string>", "exec"), scope)
+        exec(compile(code, "<string>", "exec"), scope)
         return scope["f"], tags
 
     def __get_value(self, text, scope, tag):
@@ -406,7 +404,7 @@ class _FileFromPattern(PatternFormatter):
 
     def _post(self, value, song, keep_extension=True):
         if value:
-            assert isinstance(value, text_type)
+            assert isinstance(value, str)
             value = fsnative(value)
 
             if keep_extension:
@@ -417,7 +415,7 @@ class _FileFromPattern(PatternFormatter):
                     value += ext.lower()
 
             if os.name == "nt":
-                assert isinstance(value, text_type)
+                assert isinstance(value, str)
                 value = strip_win32_incompat_from_path(value)
 
             value = expanduser(value)
@@ -433,9 +431,8 @@ class _FileFromPattern(PatternFormatter):
 class _ArbitraryExtensionFileFromPattern(_FileFromPattern):
     """Allows filename-like output with extensions different from the song."""
 
-    def _post(self, value, song):
-        super_object = super(_ArbitraryExtensionFileFromPattern, self)
-        return super_object._post(value, song, False)
+    def _post(self, value, song, keep_extension=False):
+        return super()._post(value, song, keep_extension)
 
 
 class _XMLFromPattern(PatternFormatter):
