@@ -1,6 +1,6 @@
 # Copyright 2004-2009 Joe Wreschnig, Michael Urman, Iñigo Serna,
 #                     Steven Robertson
-#           2011-2017 Nick Boultbee
+#           2011-2021 Nick Boultbee
 #           2013      Christoph Reiter
 #           2014      Jan Path
 #
@@ -62,13 +62,13 @@ class PreferencesWindow(UniqueWindow):
             def create_behaviour_frame():
                 vbox = Gtk.VBox(spacing=6)
                 jump_button = CCB(_("_Jump to playing song automatically"),
-                        'settings', 'jump', populate=True,
-                        tooltip=_("When the playing song changes, "
-                                  "scroll to it in the song list"))
+                                  'settings', 'jump', populate=True,
+                                  tooltip=_("When the playing song changes, "
+                                            "scroll to it in the song list"))
                 autosort_button = CCB(_("Sort songs when tags are modified"),
-                        'song_list', 'auto_sort', populate=True,
-                        tooltip=_("Automatically re-sort songs in the song list when "
-                                  "tags are modified"))
+                                      'song_list', 'auto_sort', populate=True,
+                                      tooltip=_("Automatically re-sort songs in "
+                                                "the song list when tags are modified"))
                 vbox.pack_start(jump_button, False, True, 0)
                 vbox.pack_start(autosort_button, False, True, 0)
                 return qltk.Frame(_("Behavior"), child=vbox)
@@ -741,7 +741,7 @@ class PreferencesWindow(UniqueWindow):
             for child in self.get_children():
                 child.show_all()
 
-    def __init__(self, parent, open_page=None):
+    def __init__(self, parent, open_page=None, all_pages=True):
         if self.is_not_unique():
             return
         super().__init__()
@@ -751,8 +751,9 @@ class PreferencesWindow(UniqueWindow):
         self.set_transient_for(qltk.get_top_parent(parent))
 
         self.__notebook = notebook = qltk.Notebook()
-        pages = [self.SongList, self.Browsers, self.Player, self.Library,
-                 self.Tagging]
+        pages = [self.Tagging]
+        if all_pages:
+            pages = [self.SongList, self.Browsers, self.Player, self.Library] + pages
         for Page in pages:
             page = Page()
             page.show()
@@ -799,6 +800,10 @@ class PreferencesWindow(UniqueWindow):
 
     def __destroy(self):
         config.save()
-        if self.current_scan_dirs != get_scan_dirs():
-            print_d("Library paths have changed, re-scanning...")
+        new_dirs = set(get_scan_dirs())
+        gone_dirs = set(self.current_scan_dirs) - new_dirs
+        if new_dirs - set(self.current_scan_dirs):
+            print_d("Library paths have been added, re-scanning...")
             scan_library(app.library, force=False)
+        elif gone_dirs:
+            copool.add(app.librarian.remove_roots, gone_dirs)
