@@ -1,13 +1,13 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman, Iñigo Serna
 #           2012 Christoph Reiter
-#           2016 Nick Boultbee
+#           2016-2020 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
+from typing import Optional, Dict, List
 import random
 
 from gi.repository import Gtk, GObject, GLib, Pango
@@ -23,7 +23,7 @@ from quodlibet.util.i18n import numeric_phrase
 from quodlibet.util.library import background_filter
 
 
-class Filter(object):
+class Filter:
 
     active_filter = None
     """A callable that returns True if the passed song should be in the
@@ -195,7 +195,7 @@ class Browser(Gtk.Box, Filter):
     the songs returned.
     """
 
-    headers = None
+    headers: Optional[List[str]] = None
     """A list of column headers to display; None means all are okay."""
 
     @classmethod
@@ -270,21 +270,24 @@ class Browser(Gtk.Box, Filter):
 
         return SongsMenu(library, songs, delete=True, items=items)
 
-    def status_text(self, count, time=None):
+    def status_text(self, count: int, time: Optional[str] = None) -> str:
         tmpl = numeric_phrase("%d song", "%d songs", count)
-        return tmpl + " (%s)" % time
+        return f"{tmpl} ({time})" if time else tmpl
 
-    replaygain_profiles = None
+    replaygain_profiles: Optional[List[str]] = None
     """Replay Gain profiles for this browser."""
 
+    def __str__(self):
+        return f"<{type(self).__name__} @ {hex(id(self))}>"
 
-class DisplayPatternMixin(object):
+
+class DisplayPatternMixin:
     """Allows Browsers customisable item (e.g. album) display patterns"""
 
     _DEFAULT_PATTERN_TEXT = ""
     """The default pattern to display"""
 
-    _PATTERN_FN = None
+    _PATTERN_FN = ""
     """The filename to save the display pattern under"""
 
     __pattern = None
@@ -303,17 +306,15 @@ class DisplayPatternMixin(object):
             cls.__pattern_text = cls._DEFAULT_PATTERN_TEXT
         cls.__refresh_pattern()
 
-    @classmethod
-    def update_pattern(cls, pattern_text):
+    def update_pattern(self, pattern_text):
         """Saves `pattern_text` to disk (and caches)"""
-        if pattern_text == cls.__pattern_text:
+        if pattern_text == self.__pattern_text:
             return
-        cls.__pattern_text = pattern_text
-        cls.__refresh_pattern()
-        cls.refresh_all()
-        print_d("Saving pattern for %s to %s" %
-                (cls.__name__, cls._PATTERN_FN))
-        with open(cls._PATTERN_FN, "w", encoding="utf-8") as f:
+        self.__pattern_text = pattern_text
+        self.__refresh_pattern()
+        self.refresh_all()
+        print_d(f"Saving pattern for {self} at {self._PATTERN_FN}")
+        with open(self._PATTERN_FN, "w", encoding="utf-8") as f:
             f.write(pattern_text + "\n")
 
     @classmethod
@@ -331,8 +332,7 @@ class DisplayPatternMixin(object):
         used for formatting entries in this browser"""
         return self.__pattern_text or self._DEFAULT_PATTERN_TEXT
 
-    @classmethod
-    def refresh_all(cls):
+    def refresh_all(self):
         """Refresh the browser's items"""
         pass
 
@@ -362,13 +362,13 @@ class FakeDisplayItem(dict):
         return value.replace("\n", ", ")
 
 
-class EditDisplayPatternMixin(object):
+class EditDisplayPatternMixin:
     """Provides a display Pattern in an editable frame"""
 
-    _PREVIEW_ITEM = None
+    _PREVIEW_ITEM: Optional[Dict[str, str]] = None
     """The `FakeItem` (or similar) to use to interpolate into the pattern"""
 
-    _DEFAULT_PATTERN = None
+    _DEFAULT_PATTERN: Optional[str] = None
     """The display pattern to use when none is saved"""
 
     def edit_display_pane(self, browser, frame_title=None):
