@@ -1,44 +1,46 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2008 Joe Wreschnig, Michael Urman, Iñigo Serna
 #           2009,2010 Steven Robertson
 #           2009-2013 Christoph Reiter
-#           2011,2013 Nick Boultbee
+#           2011-2018 Nick Boultbee
+#                2017 Fredrik Strupe
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 from gi.repository import Gtk, GLib
 
+from quodlibet import app
 from quodlibet import config
 from quodlibet import qltk
 from quodlibet import util
-
-from quodlibet.browsers._base import Browser
+from quodlibet import _
+from quodlibet.browsers import Browser
 from quodlibet.formats import PEOPLE
-from quodlibet.query import Query
+from quodlibet.qltk import is_accel
 from quodlibet.qltk.songlist import SongList
 from quodlibet.qltk.completion import LibraryTagCompletion
 from quodlibet.qltk.searchbar import SearchBarBox
 from quodlibet.qltk.x import ScrolledWindow, Align
 from quodlibet.util.library import background_filter
 from quodlibet.util import connect_destroy
+from quodlibet.qltk.paned import ConfigMultiRHPaned
 
-from .prefs import PreferencesButton
+from .prefs import PreferencesButton, ColumnMode
 from .util import get_headers
 from .pane import Pane
 
 
-class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
+class PanedBrowser(Browser, util.InstanceTracker):
     """A Browser enabling "drilling down" of tracks by successive
     selections in multiple tag pattern panes (e.g. Genre / People / Album ).
     It presents available values (and track counts) for each pane's tag
     """
 
-    __gsignals__ = Browser.__gsignals__
-
     name = _("Paned Browser")
     accelerated_name = _("_Paned Browser")
+    keys = ["Paned", "PanedBrowser"]
     priority = 3
 
     def pack(self, songpane):
@@ -53,9 +55,9 @@ class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
         container.remove(self)
 
     @classmethod
-    def set_all_wide_mode(klass, value):
+    def set_all_column_mode(klass, value):
         for browser in klass.instances():
-            browser.set_wide_mode(value)
+            browser.set_column_mode(value)
 
     @classmethod
     def set_all_panes(klass):
@@ -64,13 +66,14 @@ class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
             browser.fill_panes()
 
     def __init__(self, library):
-        super(PanedBrowser, self).__init__()
+        super().__init__()
         self._register_instance()
 
-        self._filter = None
+        self._filter = lambda s: False
         self._library = library
 
         self.set_spacing(6)
+        self.set_orientation(Gtk.Orientation.VERTICAL)
 
         completion = LibraryTagCompletion(library.librarian)
         self.accelerators = Gtk.AccelGroup()
@@ -78,12 +81,13 @@ class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
                            accel_group=self.accelerators)
         sbb.connect('query-changed', self.__text_parse)
         sbb.connect('focus-out', self.__focus)
+        sbb.connect('key-press-event', self.__sb_key_pressed)
         self._sb_box = sbb
 
         align = Align(sbb, left=6, right=6, top=6)
         self.pack_start(align, False, True, 0)
 
-        keyval, mod = Gtk.accelerator_parse("<control>Home")
+        keyval, mod = Gtk.accelerator_parse("<Primary>Home")
         self.accelerators.connect(keyval, mod, 0, self.__select_all)
         select = Gtk.Button(label=_("Select _All"), use_underline=True)
         select.connect('clicked', self.__select_all)
@@ -102,6 +106,8 @@ class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
         self.main_box = qltk.ConfigRPaned("browsers", "panedbrowser_pos", 0.4)
         self.pack_start(self.main_box, True, True, 0)
 
+        self.multi_paned = ConfigMultiRHPaned("browsers",
+                                              "panedbrowser_pane_widths")
         self.refresh_panes()
 
         for child in self.get_children():
@@ -110,17 +116,19 @@ class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
     def __destroy(self, *args):
         del self._sb_box
 
-    def set_wide_mode(self, do_wide):
+    def set_column_mode(self, mode):
         hor = Gtk.Orientation.HORIZONTAL
         ver = Gtk.Orientation.VERTICAL
-        panes = self.main_box.get_child1()
 
-        if do_wide:
+        if mode == ColumnMode.WIDE:
             self.main_box.props.orientation = hor
-            panes.props.orientation = ver
-        else:
+            self.multi_paned.change_orientation(horizontal=False)
+        elif mode == ColumnMode.COLUMNAR:
+            self.main_box.props.orientation = hor
+            self.multi_paned.change_orientation(horizontal=True)
+        else:  # ColumnMode.SMALL
             self.main_box.props.orientation = ver
-            panes.props.orientation = hor
+            self.multi_paned.change_orientation(horizontal=True)
 
     def _get_text(self):
         return self._sb_box.get_text()
@@ -134,10 +142,21 @@ class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
     def __text_parse(self, bar, text):
         self.activate()
 
+    def __sb_key_pressed(self, entry, event):
+        if (is_accel(event, "<Primary>Return") or
+                is_accel(event, "<Primary>KP_Enter")):
+            songs = app.window.songlist.get_songs()
+            app.window.playlist.enqueue(songs)
+            return True
+        return False
+
     def filter_text(self, text):
         self._set_text(text)
         self.activate()
 
+    def get_filter_text(self):
+        return self._get_text()
+
     def __select_all(self, *args):
         self._panes[-1].inhibit()
         for pane in self._panes:
@@ -146,13 +165,13 @@ class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
         self._panes[-1].get_selection().emit('changed')
 
     def __added(self, library, songs):
-        songs = filter(self._filter, songs)
+        songs = list(filter(self._filter, songs))
         for pane in self._panes:
             pane.add(songs)
-            songs = filter(pane.matches, songs)
+            songs = list(filter(pane.matches, songs))
 
     def __removed(self, library, songs, remove_if_empty=True):
-        songs = filter(self._filter, songs)
+        songs = list(filter(self._filter, songs))
         for pane in self._panes:
             pane.remove(songs, remove_if_empty)
 
@@ -174,15 +193,15 @@ class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
         return True
 
     def activate(self):
-        text = self._get_text()
-        if Query.is_parsable(text):
-            star = dict.fromkeys(SongList.star)
-            star.update(self.__star)
-            self._filter = Query(text, star.keys()).search
-            songs = filter(self._filter, self._library)
+        star = dict.fromkeys(SongList.star)
+        star.update(self.__star)
+        query = self._sb_box.get_query(star.keys())
+        if query.is_parsable:
+            self._filter = query.search
+            songs = list(filter(self._filter, self._library))
             bg = background_filter()
             if bg:
-                songs = filter(bg, songs)
+                songs = list(filter(bg, songs))
             self._panes[0].fill(songs)
 
     def scroll(self, song):
@@ -190,54 +209,56 @@ class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
             pane.scroll(song)
 
     def refresh_panes(self):
-        hbox = self.main_box.get_child1()
-        if hbox:
-            hbox.destroy()
-
-        hbox = Gtk.HBox(spacing=6)
-        hbox.set_homogeneous(True)
+        self.multi_paned.destroy()
 
         # Fill in the pane list. The last pane reports back to us.
         self._panes = [self]
         for header in reversed(get_headers()):
             pane = Pane(self._library, header, self._panes[0])
+            pane.connect('row-activated',
+                         lambda *x: self.songs_activated())
             self._panes.insert(0, pane)
         self._panes.pop()  # remove self
 
+        # Put the panes in scrollable windows
+        sws = []
         for pane in self._panes:
-            pane.connect('row-activated',
-                         lambda *x: self.songs_activated())
             sw = ScrolledWindow()
             sw.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
             sw.set_shadow_type(Gtk.ShadowType.IN)
             sw.add(pane)
-            hbox.pack_start(sw, True, True, 0)
+            sws.append(sw)
 
-        self.main_box.pack1(hbox, True, False)
-        hbox.show_all()
+        self.multi_paned.set_widgets(sws)
+        self.multi_paned.show_all()
+        self.main_box.pack1(self.multi_paned.get_paned(), True, False)
 
         self.__star = {}
         for p in self._panes:
             tags = [t for t in p.tags if not t.startswith("~#")]
             self.__star.update(dict.fromkeys(tags))
 
-        self.set_wide_mode(config.getboolean("browsers", "pane_wide_mode"))
+        self.set_column_mode(config.getint("browsers", "pane_mode",
+                                           ColumnMode.SMALL))
 
     def fill_panes(self):
         self._panes[-1].inhibit()
         self.activate()
         self._panes[-1].uninhibit()
 
+    def make_pane_widths_equal(self):
+        self.multi_paned.make_pane_widths_equal()
+
     def __get_filter_pane(self, key):
         """Get the best pane for filtering etc."""
 
-        canditates = []
+        candidates = []
         for pane in self._panes:
             if (key in pane.tags or
                     (key in PEOPLE and "~people" in pane.tags)):
-                canditates.append((len(pane.tags), pane))
-        canditates.sort()
-        return (canditates and canditates[0][1]) or None
+                candidates.append((len(pane.tags), pane))
+        candidates.sort()
+        return (candidates and candidates[0][1]) or None
 
     def can_filter_tag(self, tag):
         return (self.__get_filter_pane(tag) is not None)
@@ -251,6 +272,7 @@ class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
         for pane in self._panes:
             if pane is filter_pane:
                 filter_pane.set_selected_by_tag(tag, values, True)
+                filter_pane.grab_focus()
                 return
             pane.set_selected([None], True)
 
@@ -266,7 +288,7 @@ class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
         filter_pane = self.__get_filter_pane(key)
 
         if filter_pane is None:
-            return super(PanedBrowser, self).list(key)
+            return super().list(key)
 
         for pane in self._panes:
             if pane is filter_pane:
@@ -275,39 +297,32 @@ class PanedBrowser(Gtk.VBox, Browser, util.InstanceTracker):
         return []
 
     def save(self):
-        config.set("browsers", "query_text", self._get_text())
+        config.settext("browsers", "query_text", self._get_text())
 
         selected = []
         for pane in self._panes:
             selected.append(pane.get_restore_string())
 
-        to_save = u"\n".join(selected).encode("utf-8")
-        config.set("browsers", "pane_selection", to_save)
+        to_save = u"\n".join(selected)
+        config.settext("browsers", "pane_selection", to_save)
 
     def restore(self):
         try:
-            text = config.get("browsers", "query_text")
+            text = config.gettext("browsers", "query_text")
         except config.Error:
             pass
         else:
             self._set_text(text)
 
-        selected = config.get("browsers", "pane_selection")
+        selected = config.gettext("browsers", "pane_selection")
         if not selected:
             return
 
-        try:
-            selected = selected.decode("utf-8")
-        except UnicodeDecodeError:
-            return
-
         for pane, string in zip(self._panes, selected.split(u"\n")):
             pane.parse_restore_string(string)
 
     def finalize(self, restored):
-        config.set("browsers", "query_text", "")
-        if not restored:
-            self.fill_panes()
+        config.settext("browsers", "query_text", u"")
 
     def fill(self, songs):
         GLib.idle_add(self.songs_selected, list(songs))
