@@ -1,19 +1,20 @@
-# -*- coding: utf-8 -*-
-# Copyright 2012 - 2014 Christoph Reiter, Nick Boultbee
+# Copyright 2012 - 2020 Christoph Reiter, Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
+from typing import Optional, Iterable
+
 from quodlibet import _
 from quodlibet import config
 from quodlibet import util
-from quodlibet.util.modulescanner import ModuleScanner
-from quodlibet.util.dprint import print_d
-from quodlibet.util.config import ConfigProxy
 from quodlibet.qltk.ccb import ConfigCheckButton
-from quodlibet.compat import itervalues, iteritems, listkeys, string_types
+from quodlibet.util import escape
+from quodlibet.util.config import ConfigProxy
+from quodlibet.util.dprint import print_d
+from quodlibet.util.modulescanner import ModuleScanner
 
 
 def init(folders=None, disable_plugins=False):
@@ -37,7 +38,7 @@ class PluginImportException(Exception):
     desc = ""
 
     def __init__(self, desc, *args, **kwargs):
-        super(PluginImportException, self).__init__(desc)
+        super().__init__(desc)
         self.desc = desc
 
     def should_show(self):
@@ -51,7 +52,7 @@ class PluginNotSupportedError(PluginImportException):
 
     def __init__(self, msg=None):
         msg = "not supported: %s" % (msg or "unknown reason")
-        super(PluginNotSupportedError, self).__init__(msg)
+        super().__init__(msg)
 
     def should_show(self):
         return False
@@ -62,7 +63,7 @@ class MissingModulePluginException(PluginImportException):
     def __init__(self, module_name):
         msg = (_("Couldn't find module '{module}'. Perhaps you need to "
                  "install the package?").format(module=module_name))
-        super(MissingModulePluginException, self).__init__(msg)
+        super().__init__(msg)
 
 
 class MissingGstreamerElementPluginException(PluginImportException):
@@ -71,7 +72,7 @@ class MissingGstreamerElementPluginException(PluginImportException):
     def __init__(self, element_name):
         msg = (_("Couldn't find GStreamer element '{element}'.")
                  .format(element=element_name))
-        super(MissingGstreamerElementPluginException, self).__init__(msg)
+        super().__init__(msg)
 
 
 def migrate_old_config():
@@ -114,7 +115,7 @@ def list_plugins(module):
     return ok
 
 
-class PluginModule(object):
+class PluginModule:
 
     def __init__(self, name, module):
         self.name = name
@@ -122,7 +123,7 @@ class PluginModule(object):
         self.plugins = [Plugin(cls) for cls in list_plugins(module)]
 
 
-class Plugin(object):
+class Plugin:
 
     def __init__(self, plugin_cls):
         self.cls = plugin_cls
@@ -148,10 +149,17 @@ class Plugin(object):
     def description(self):
         return getattr(self.cls, "PLUGIN_DESC", None)
 
+    @property
+    def description_markup(self):
+        try:
+            return getattr(self.cls, "PLUGIN_DESC_MARKUP")
+        except AttributeError:
+            return escape(self.description)
+
     @property
     def tags(self):
         tags = getattr(self.cls, "PLUGIN_TAGS", [])
-        if isinstance(tags, string_types):
+        if isinstance(tags, str):
             tags = [tags]
         return tags
 
@@ -176,7 +184,7 @@ class Plugin(object):
         return self.instance
 
 
-class PluginHandler(object):
+class PluginHandler:
     """A plugin handler can choose to handle plugins, as well as control
     their enabled state."""
 
@@ -193,7 +201,7 @@ class PluginHandler(object):
         raise NotImplementedError
 
 
-class PluginManager(object):
+class PluginManager:
     """
     The manager takes care of plugin loading/reloading. Interested plugin
     handlers can register them self to get called when plugins get enabled
@@ -226,14 +234,15 @@ class PluginManager(object):
     CONFIG_SECTION = "plugins"
     CONFIG_OPTION = "active_plugins"
 
-    instance = None  # default instance
+    instance: Optional["PluginManager"] = None
+    """Default instance"""
 
     def __init__(self, folders=None):
         """folders is a list of paths that will be scanned for plugins.
         Plugins in later paths will be preferred if they share a name.
         """
 
-        super(PluginManager, self).__init__()
+        super().__init__()
 
         if folders is None:
             folders = []
@@ -278,17 +287,14 @@ class PluginManager(object):
 
     @property
     def _modules(self):
-        return itervalues(self.__scanner.modules)
+        return self.__scanner.modules.values()
 
     @property
-    def _plugins(self):
+    def _plugins(self) -> Iterable[Plugin]:
         """All registered plugins"""
-
-        plugins = []
-        for module in itervalues(self.__modules):
-            for plugin in module.plugins:
-                plugins.append(plugin)
-        return plugins
+        return (plugin
+                for module in self.__modules.values()
+                for plugin in module.plugins)
 
     @property
     def plugins(self):
@@ -369,7 +375,7 @@ class PluginManager(object):
         """module name: list of error message text lines"""
 
         errors = {}
-        for name, error in iteritems(self.__scanner.failures):
+        for name, error in self.__scanner.failures.items():
             exception = error.exception
             if isinstance(exception, PluginImportException):
                 if not exception.should_show():
@@ -383,7 +389,7 @@ class PluginManager(object):
     def quit(self):
         """Disable plugins and tell all handlers to clean up"""
 
-        for name in listkeys(self.__modules):
+        for name in list(self.__modules.keys()):
             self.__remove_module(name)
 
     def __remove_module(self, name):
@@ -440,7 +446,7 @@ class PluginConfig(ConfigProxy):
         self._prefix = prefix
         if _config is None:
             _config = config._config
-        super(PluginConfig, self).__init__(
+        super().__init__(
             _config, PM.CONFIG_SECTION, _defaults)
 
     def _new_defaults(self, real_default_config):
@@ -454,7 +460,7 @@ class PluginConfig(ConfigProxy):
                                  self._option(option), **kwargs)
 
 
-class PluginConfigMixin(object):
+class PluginConfigMixin:
     """
     Mixin for storage and editing of plugin config in a standard way.
     """
@@ -519,7 +525,7 @@ class PluginConfigMixin(object):
                                  option, populate=True)
 
 
-class ConfProp(object):
+class ConfProp:
 
     def __init__(self, conf, name, default):
         self._conf = conf
