@@ -1,11 +1,11 @@
-# -*- coding: utf-8 -*-
 # Copyright 2005 Joe Wreschnig, Michael Urman
-#           2013 Nick Boultbee
+#           2013-2017 Nick Boultbee
 #           2013,2014 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 """
 Functions for deleting files and songs with user interaction.
@@ -16,24 +16,28 @@ Only use trash_files() or trash_songs() and TrashMenuItem().
 import os
 
 from gi.repository import Gtk
+from senf import fsn2text
 
+from quodlibet import _
+from quodlibet import print_w
 from quodlibet.util import trash
 from quodlibet.qltk import get_top_parent
 from quodlibet.qltk import Icons
 from quodlibet.qltk.msg import ErrorMessage, WarningMessage
 from quodlibet.qltk.wlw import WaitLoadWindow
 from quodlibet.qltk.x import MenuItem, Align
-from quodlibet.util.path import fsdecode, unexpand
+from quodlibet.util.i18n import numeric_phrase
+from quodlibet.util.path import unexpand
 
 
 class FileListExpander(Gtk.Expander):
     """A widget for showing a static list of file paths"""
 
     def __init__(self, paths):
-        super(FileListExpander, self).__init__(label=_("Files:"))
+        super().__init__(label=_("Files:"))
         self.set_resize_toplevel(True)
 
-        paths = [fsdecode(unexpand(p)) for p in paths]
+        paths = [fsn2text(unexpand(p)) for p in paths]
         lab = Gtk.Label(label="\n".join(paths))
         lab.set_alignment(0.0, 0.0)
         lab.set_selectable(True)
@@ -49,7 +53,7 @@ class FileListExpander(Gtk.Expander):
 class DeleteDialog(WarningMessage):
 
     RESPONSE_DELETE = 1
-    """"Return value of DeleteDialog.run() in case the passed files
+    """Return value of DeleteDialog.run() in case the passed files
     should be deleted"""
 
     @classmethod
@@ -69,14 +73,11 @@ class DeleteDialog(WarningMessage):
         return cls(parent, paths, description)
 
     def __init__(self, parent, paths, description):
-        title = ngettext(
-            "Delete %(file_count)d file permanently?",
-            "Delete %(file_count)d files permanently?",
-            len(paths)) % {
-                "file_count": len(paths),
-            }
+        title = numeric_phrase("Delete %(file_count)d file permanently?",
+                               "Delete %(file_count)d files permanently?",
+                               len(paths), "file_count")
 
-        super(DeleteDialog, self).__init__(
+        super().__init__(
             get_top_parent(parent),
             title, description,
             buttons=Gtk.ButtonsType.NONE)
@@ -95,7 +96,7 @@ class DeleteDialog(WarningMessage):
 class TrashDialog(WarningMessage):
 
     RESPONSE_TRASH = 1
-    """"Return value of TrashDialog.run() in case the passed files
+    """Return value of TrashDialog.run() in case the passed files
     should be moved to the trash"""
 
     @classmethod
@@ -116,14 +117,10 @@ class TrashDialog(WarningMessage):
 
     def __init__(self, parent, paths, description):
 
-        title = ngettext(
-            "Move %(file_count)d file to the trash?",
-            "Move %(file_count)d files to the trash?",
-            len(paths)) % {
-                "file_count": len(paths),
-            }
-
-        super(TrashDialog, self).__init__(
+        title = numeric_phrase("Move %(file_count)d file to the trash?",
+                               "Move %(file_count)d files to the trash?",
+                               len(paths), "file_count")
+        super().__init__(
             get_top_parent(parent),
             title, description,
             buttons=Gtk.ButtonsType.NONE)
@@ -163,7 +160,8 @@ def _do_trash_songs(parent, songs, librarian):
         filename = song("~filename")
         try:
             trash.trash(filename)
-        except trash.TrashError:
+        except trash.TrashError as e:
+            print_w("Couldn't trash file (%s)" % e)
             failed.append(song)
         else:
             ok.append(song)
